require 'set'

require 'svg_optimizer'  # https://github.com/fnando/svg_optimizer

require 'distorted/checking_you_out'
require 'distorted/modular_technology/vips/save'


module Cooltrainer; end
module Cooltrainer::DistorteD; end
module Cooltrainer::DistorteD::Molecule; end
module Cooltrainer::DistorteD::Molecule::SVG

  include Cooltrainer::DistorteD::Technology::Vips::Save

  LOWER_WORLD = Hash[
    CHECKING::YOU::OUT['image/svg+xml'] => Cooltrainer::DistorteD::Technology::Vips::vips_get_options(
      Cooltrainer::DistorteD::Technology::Vips::vips_foreign_find_load_suffix(".#{CHECKING::YOU::OUT['image/svg+xml'].preferred_extension}")
    ).merge(Hash[
      :optimize => Cooltrainer::Compound.new(:optimize, valid: Cooltrainer::BOOLEAN_VALUES, default: false, blurb: 'SvgOptimizer'),
    ])
  ]

  # WISHLIST: Support VML for old IE compatibility.
  #  Example: RaphaëlJS — https://en.wikipedia.org/wiki/Rapha%C3%ABl_(JavaScript_library)
  OUTER_LIMITS = Hash[
    CHECKING::YOU::OUT['image/svg+xml'] => nil,
  ]

  def to_vips_image
    # NOTE: libvips 8.9 added the `unlimited` argument to svgload.
    # Loading SVGs >= 10MiB in size will fail on older libvips.
    # https://github.com/libvips/libvips/commit/55e49831b801e05ddd974b1e2102fda7956c53f5
    @vips_image ||= Vips::Image.new_from_file(path)
  end

  define_method(CHECKING::YOU::OUT['image/svg+xml'].distorted_file_method) { |dest_root, change|
    if change.optimize
      SvgOptimizer.optimize_file(path, change.paths(dest_root).first, SvgOptimizer::DEFAULT_PLUGINS)
    else
      copy_file(change.paths(dest_root).first)
    end
  }

end
